<a href="https://www.numfocus.org">
  <image src="https://img.shields.io/badge/powered%20by-NumFOCUS-orange.svg?style=flat&amp;colorA=E1523D&amp;colorB=007D8A">
</a>

<br/>

<a href="http://depsy.org/package/python/matplotlib">
  <img src="{{ pathto('_static/depsy_badge.svg', 1) }}">
</a>

<br/>

Travis-CI: <a href="https://travis-ci.org/matplotlib/matplotlib">
  <img src="https://travis-ci.org/matplotlib/matplotlib.svg?branch=master"/>
</a>
<br/>
