{% extends "layout.html" %}
{% set title = "Citing Matplotlib" %}
{% block body %}

<h1>Citing Matplotlib</h1>
<p>
If Matplotlib contributes to a project that leads to a scientific publication,
please acknowledge this fact by citing the project. You can use this
BibTeX entry:
</p>
<pre>
@Article{Hunter:2007,
  Author    = {Hunter, J. D.},
  Title     = {Matplotlib: A 2D graphics environment},
  Journal   = {Computing In Science \& Engineering},
  Volume    = {9},
  Number    = {3},
  Pages     = {90--95},
  abstract  = {Matplotlib is a 2D graphics package used for Python
  for application development, interactive scripting, and
  publication-quality image generation across user
  interfaces and operating systems.},
  publisher = {IEEE COMPUTER SOC},
  doi = {10.1109/MCSE.2007.55},
  year      = 2007
}
</pre>


<h2>DOIs</h2>
<dl>
  <dt>v2.0.2</dt> <dd><a href="https://doi.org/10.5281/zenodo.573577"><img src="https://zenodo.org/badge/DOI/10.5281/zenodo.573577.svg" alt="DOI"></a></dd>
  <dt>v2.0.1</dt> <dd><a href="https://doi.org/10.5281/zenodo.570311"><img src="https://zenodo.org/badge/DOI/10.5281/zenodo.570311.svg" alt="DOI"></a></dd>
  <dt>v2.0.0</dt><dd><a href="https://doi.org/10.5281/zenodo.248351"><img src="https://zenodo.org/badge/DOI/10.5281/zenodo.248351.svg" alt="DOI"></a></dd>
  <dt>v1.5.3</dt><dd><a href="http://dx.doi.org/10.5281/zenodo.61948"><img src="https://zenodo.org/badge/doi/10.5281/zenodo.61948.svg" alt="10.5281/zenodo.61948"></a></dd>
  <dt>v1.5.2</dt><dd><a href="http://dx.doi.org/10.5281/zenodo.56926"><img src="https://zenodo.org/badge/doi/10.5281/zenodo.56926.svg" alt="10.5281/zenodo.56926"></a></dd>
  <dt>v1.5.1</dt><dd><a href="http://dx.doi.org/10.5281/zenodo.44579"><img src="https://zenodo.org/badge/doi/10.5281/zenodo.44579.svg" alt="10.5281/zenodo.44579"></a></dd>
  <dt>v1.5.0</dt><dd><a href="http://dx.doi.org/10.5281/zenodo.32914"><img src="https://zenodo.org/badge/doi/10.5281/zenodo.32914.svg" alt="10.5281/zenodo.32914"></a></dd>
  <dt>v1.4.3</dt><dd><a href="http://dx.doi.org/10.5281/zenodo.15423"><img src="https://zenodo.org/badge/doi/10.5281/zenodo.15423.svg" alt="10.5281/zenodo.15423"></a></dd>
  <dt>v1.4.2</dt><dd><a href="http://dx.doi.org/10.5281/zenodo.12400"><img src="https://zenodo.org/badge/doi/10.5281/zenodo.12400.png" alt="10.5281/zenodo.12400"></a></dd>
  <dt>v1.4.1</dt><dd><a href="http://dx.doi.org/10.5281/zenodo.12287"><img src="https://zenodo.org/badge/doi/10.5281/zenodo.12287.png" alt="10.5281/zenodo.12287"></a></dd>
  <dt>v1.4.0</dt><dd><a href="http://dx.doi.org/10.5281/zenodo.11451"><img src="https://zenodo.org/badge/doi/10.5281/zenodo.11451.png" alt="10.5281/zenodo.11451"></a></dd>
</dl>

{% endblock %}
